startID = 0
stopID = 10

--logFilePath = "C:\FEMMExport-CCTest"..startID.."-"..stopID..".csv"
--logFilePath = "Z:/Desktop/ROM5-"..startID.."-"..stopID..".csv"
logFilePath = "Z:/Desktop/MagOD-ConstV2.csv"

--====================================================================================================
--		TEST PARAMETERS			TEST PARAMETERS			TEST PARAMETERS			TEST PARAMETERS
--==================================================================================================== 
borderNodes   = {{0, 1.7},{0,-1.7}}	-- The coordinates of the border nodes (inches)
mag2coilSpace = 0.014 					-- Air gap beween magnet and coil (inches)
coil2shieldSpace = 0.002				-- Glue gap between coil and magnetic shield steel
shieldThickness = 0.015					-- Thickness of 1020 Steel shield
shieldLength = 1.500					-- Shield length
magTotalTravel = 0.5					-- Travel of magnet to analyze (+/- magTotalTravel/2)
magTravelStep = 0.025					-- Travel Step Increment (inches)
magOvershoot = 0.125					-- how far past the magnet centers we will analyze for each situation

voltage,	voltage_min	, voltage_max, voltage_step = 0,  15,      15, 1		 	-- 1- Power rail voltage (4 combinations)
wireAWG,	wireAWG_min	, wireAWG_max, wireAWG_step = 0,  36,          36, 1				-- 1- Wire Gauge (8 combinations)
coilCC,		coilCC_min	, coilCC_max , coilCC_step	= 0,  0.5,      0.5, 1    		-- 9- Coil separation, center-to-center
magID,		magID_min	, magID_max	 , magID_step	= 0,  0.125,  0.125 , 1 	-- 1- Magnet Inner Diameter
magOD,		magOD_min	, magOD_max	 , magOD_step	= 0,  0.250,    0.25, 0.125  	-- 1- Magnet Outer Diameter
coilOD,		coilOD_min	, coilOD_max , coilOD_step	= 0,  0.35,      0.35, 1  		-- 1- Coil Outer Diameter
coilLen,	coilLen_min	, coilLen_max, coilLen_step	= 0,  0.5,      0.5, 1			-- 1- Coil Length

--magLen, magLen_step = 0.5, .125 -- see description below about designing the magnet length

--Magnet length equal to coilCC will generate the highest peak force for center position. 
--Increasing magnet length up to 2x coilCC will flatten the force curve, but also extend the flat range past the center points of the coils if desired.
--Always start with magLen=coilCC, take it to magLen =coilCC*2 and choose desired force curve shape
magLen,		magLen_min	, magLen_max , magLen_step	= 0,  0.25,  0.75, 0.125 	 -- 1- Magnet Length

forcedAmperes = 0 -- set this to anything but zero to force the simulation to run at this current instead of the calculated current based on voltage
forcedPower = 0	-- set this to anything but zero to force the simulation to run at this power instead of calculated or forcedAmperes

coilID = 0 		-- this is calculated based on magnet OD
numTurns = 0 	-- this is calculated based on coil dimensions and AWG
amperes = 0		-- this is calculated based on coil resistance and drive voltage
resistance = 0	-- this is calculated based on coil current/voltage after the initial analysis iteration

-- Total Combinations = 2*6*4*3*2*7*4*5  =  40320
-- Total Data Points @ 28 points per combination = 1128960
-- Estimated Processing Time @ 1.5 s/combination = 13days - oh shit.

--====================================================================================================
--		VARIABLE DECLARATIONS		VARIABLE DECLARATIONS		VARIABLE DECLARATIONS
--==================================================================================================== 
configID = 0	-- Each actuator configuration is assigned an incremental numeric ID during the loop

magnetNodes = {{0,0},{0,0},{0,0},{0,0}}	-- stores the (r,z) coordinates of the four points defining the magnet
coilANodes 	= {{0,0},{0,0},{0,0},{0,0}}	-- stores the (r,z) coordinates of the four points defining Coil A
coilBNodes	= {{0,0},{0,0},{0,0},{0,0}}	-- stores the (r,z) coordinates of the four points defining Coil B
shieldNodes = {{0,0},{0,0},{0,0},{0,0}}	-- stores the (r,z) coordinates of the four points defining the shield

magnetLabel = {0,0} -- coordinates of the magnet materials label (default)
coilALabel =  {0,0} -- coordinates of the coil A material label  (default)
coilBLabel =  {0,0} -- coordinates of the coil B material label  (default)
shieldLabel = {0,0} -- coordinates of the shield (default)
progressLabel={0,0} -- coordinates of the progress label (shows user what configuration # we're processing)

magnetGroup = 1
coilAGroup  = 2
coilBGroup  = 3
shieldGroup = 4

--Arrays are INDEX 1!! - WTF?
AWGIndexOffset = 13 --  (i.e, wireOD[AWG-AWGIndexOffset] = wire diameter of AWG wire. wireOD[1] is for 14AWG)

-- wire OD with enamel/coating (inches){
wireOD = {	0.0682, 0.0610,	0.0545,	0.0488,	0.0437,	0.0391,	0.0351,	0.0315,	0.0281,	0.0253,	0.0227,
			0.0203,	0.0182,	0.0164,	0.0147,	0.0133,	0.0121,	0.0108,	0.0097,	0.0087,	0.0078,	0.0070,
			0.0063,	0.0057,	0.0051,	0.0045,	0.0041,	0.0037,	0.0033,	0.0029,	0.0026,	0.0024,	0.0021,
			0.0019,	0.0017,	0.0015,	0.0014
		 }

-- wire OD without enamel/coating (conductor only, inches)
condOD = {	0.0635, 0.0565, 0.0503, 0.0448, 0.0399, 0.0355, 0.0317, 0.0282, 0.0250, 0.0224, 0.0199,
			0.0177, 0.0157, 0.0141, 0.0125, 0.0112, 0.0099, 0.0088, 0.0079, 0.0070, 0.0062, 0.0055,
			0.0049, 0.0044, 0.0039, 0.0034, 0.0030, 0.0027, 0.0024, 0.0021, 0.0019, 0.00169, 0.00151,
			0.00135, 0.00119, 0.00107, 0.00095
		 }

--====================================================================================================
--		SUPPORT FUNCTIONS		SUPPORT FUNCTIONS		SUPPORT FUNCTIONS		SUPPORT FUNCTIONS
--==================================================================================================== 
 
function drawBoundary()
	mi_addnode(borderNodes[1][1], borderNodes[1][2])
	mi_addnode(borderNodes[2][1], borderNodes[2][2])
	
	mi_addsegment(borderNodes[1][1], borderNodes[1][2], borderNodes[2][1], borderNodes[2][2])
	mi_addarc(borderNodes[2][1], borderNodes[2][2], borderNodes[1][1], borderNodes[1][2], 180, 1)
	
	mi_addblocklabel(borderNodes[1][2]-0.25, 0) -- set the space inside the boundary as AIR
	mi_selectlabel(borderNodes[1][2]-0.25, 0)
	mi_setblockprop("Air",1,0,"<none>",0,0,0)
	
	mi_addblocklabel(borderNodes[1][2]+0.01,0) -- insert the configuration ID label so that we can tell which config we're processing
	mi_clearselected()
	mi_selectlabel(borderNodes[1][2]+0.01,0)
	progressLabel = { borderNodes[1][2]+0.01,0 }
	mi_setblockprop("Configuration # "..startID,1,0,"<none>",0,0,0)
	
	mi_selectarcsegment(borderNodes[2][1]+0.01, borderNodes[2][2])
	mi_setarcsegmentprop(1,"ABC",0,0)
	mi_selectsegment(borderNodes[2][1], borderNodes[2][2])
	mi_setsegmentprop("ABC",0,1,0,0)
end
function init()
	-- set the problem to inches, axisymmetric, precision of 10^-8 (minimum precision)
	mi_probdef(0,"inches","axi", 1e-008)
	
	-- get materials from library
	--mi_getmaterial(magnet_mat) -- we are making a custom magnetic material below, so we comment this line out
	mi_getmaterial("Air")
	mi_getmaterial("1020 Steel") -- for the shield
	
	-- create border condition
	mi_addboundprop("ABC",0,0,0,0,0,0,7832428.301766503,0,2)
	
	-- create the two circuit definitions
	mi_addcircprop("COILA",  1, 1)  -- define circuit A, +1A, 1=Series circuit
	mi_addcircprop("COILB", -1, 1)  -- define circuit B, -1A,  1=Series circuit
	
	--make generic wire material
	mi_addmaterial("WIRE_CUST",1,1,0,0,58,0,0,1,3,0,0,1,condOD[36-AWGIndexOffset]*25.4) -- default to 36AWG wire
	mi_addmaterial("MAG_CUST" ,1.049,1.049,1006582,0,0,0,0,0,0,0,0,0) 
	mi_addmaterial("Configuration # "..startID,1,1,0,0,0,0,0,1,3,0,0,1,0.1) -- label that shows the user which iteration we're processing
	
	drawBoundary() -- draw the problem boundary / add boundary conditions
	mi_saveas("./test.fem")
end
function drawRect(nodes, material, circuit, group, turns, labelCoords)
	-- add the four nodes
	mi_addnode(nodes[1][1], nodes[1][2]) --point 1
	mi_selectnode(nodes[1][1], nodes[1][2])
	mi_setnodeprop("<none>",group)
	
	mi_addnode(nodes[2][1], nodes[2][2]) --point 2
	mi_selectnode(nodes[2][1], nodes[2][2])
	mi_setnodeprop("<none>",group)
	
	mi_addnode(nodes[3][1], nodes[3][2]) --point 3	
	mi_selectnode(nodes[3][1], nodes[3][2])
	mi_setnodeprop("<none>",group)
	
	mi_addnode(nodes[4][1], nodes[4][2]) --point 4
	mi_selectnode(nodes[4][1], nodes[4][2])
	mi_setnodeprop("<none>",group)
	
	-- draw the four segments
	mi_addsegment( nodes[1][1], nodes[1][2], nodes[2][1], nodes[2][2]) -- 1 > 2
	mi_addsegment( nodes[2][1], nodes[2][2], nodes[3][1], nodes[3][2]) -- 2 > 3
	mi_addsegment( nodes[3][1], nodes[3][2], nodes[4][1], nodes[4][2]) -- 3 > 4
	mi_addsegment( nodes[4][1], nodes[4][2], nodes[1][1], nodes[1][2]) -- 4 > 1
	
	--load the material from the library, add block label and set block material/properties
	
	labelCoords[1] = (nodes[1][1]+nodes[3][1])/2
	labelCoords[2] = (nodes[1][2]+nodes[3][2])/2
	mi_addblocklabel( labelCoords[1],labelCoords[2] )
	mi_selectlabel( labelCoords[1], labelCoords[2] )
	mi_setblockprop(material,1,0,circuit,90,group,turns)

end
function defineRectNodes(drn_id, drn_od, drn_len)
	-- defines the magnet node positions based on the specified magnet geometry
	drn_nodes = { 	
					{drn_id/2, drn_len/2},
					{drn_id/2,-drn_len/2},
					{drn_od/2,-drn_len/2},
					{drn_od/2, drn_len/2}
				}
	return drn_nodes
end
function defineCoilNodes(coil_id, coil_od, coil_length, coil_offset)
	-- defines the coil node positions based on the specified coil geometries
	coilANodes = {
					{coil_id/2, coil_offset/2+coil_length/2},
					{coil_id/2, coil_offset/2-coil_length/2},
					{coil_od/2, coil_offset/2-coil_length/2},
					{coil_od/2, coil_offset/2+coil_length/2}
				 }
			
	coilBNodes = {
					{coil_id/2, -coil_offset/2-coil_length/2},
					{coil_id/2, -coil_offset/2+coil_length/2},
					{coil_od/2, -coil_offset/2+coil_length/2},
					{coil_od/2, -coil_offset/2-coil_length/2}
				 }

end
function defineCoilSpecs(coil_id, coil_od, coil_length, wire_awg)
	-- determines the maximum number of windings possible with specified dimensions and wire gauge
	-- returns the number of turns and resulting coil OD (based on physical windings not exceeding defined coil OD)
	windingsPerLayer = floor(((coil_length)/(wireOD[wire_awg-AWGIndexOffset]*1.1)))
	layersPossible = floor(((coil_od-coil_id)/2/(wireOD[wire_awg-AWGIndexOffset]*1.1)))
	numTurns = windingsPerLayer*layersPossible
	actualOD = coil_id + layersPossible*(wireOD[wire_awg-AWGIndexOffset]/1.1)*2
	return numTurns, actualOD, layersPossible
end

--====================================================================================================
--		MAIN PROGRAM		MAIN PROGRAM		MAIN PROGRAM		MAIN PROGRAM		MAIN PROGRAM
--====================================================================================================

newdocument(0) --create/open new magnetics problem file
init() 		   -- initialize the FEMM environment boundaries, materials, circuits

--Create the data file and write the file header into the first line/row
local file = openfile(logFilePath, "w") -- create the data file
write(file,"ConfigID,Voltage,Current,Resistance,Power,AWG,#Turns,Layers,RangeOfMotion,MagnetID,MagnetOD,MagnetLen,CoilID,CoilOD,CoilLen")

-- write the x-axis points into the header row
for temp1 = -magTotalTravel/2, magTotalTravel/2, magTravelStep do
	write(file,",",temp1)
end
write(file,"\n")
-- end of the header row


--The main loop block (I didn't bother with proper nest formatting, but they are nested)
for voltage = voltage_min,	voltage_max, voltage_step	do
for wireAWG = wireAWG_min,	wireAWG_max, wireAWG_step	do
for coilCC  = coilCC_min,	coilCC_max,  coilCC_step	do
for magID   = magID_min,	magID_max,   magID_step		do
for magOD 	= magOD_min,	magOD_max,   magOD_step		do
for magLen 	= magLen_min,		magLen_max,  	 magLen_step	do
for coilOD 	= coilOD_min,	coilOD_max,  wireOD[wireAWG-AWGIndexOffset]	do -- step the coilOD according to the wire OD (add layers)
for coilLen = coilLen_min,	coilLen_max, coilLen_step	do
	coilLen = coilCC	
	oldcoilOD = coilOD
	coilID = magOD + mag2coilSpace*2	-- the coil ID is always a set distance from the magnet OD
	numTurns, coilOD, numLayers = defineCoilSpecs(coilID, coilOD, coilLen, wireAWG) --coilOD gets overwritten to the exact value of the OD based on the wire winding layer thickness
	
	-- Check for, and skip, any invalid configurations
	validConfig = 1
	if(magOD <= magID) then 
		validConfig = 0 
	end	-- Magnets whose ID exceeds or matches the OD
	
	if(coilLen > coilCC) then 
		validConfig = 0 
		--messagebox("invalid")
	end 	-- Coils that overlap in the center
	
	if(numTurns < 1) then
		validConfig = 0
	end	-- Coils with less than one turn
	
	if(coilID >= coilOD) then 
		validConfig = 0 
	end	-- Coils whose ID exceeds or matches their OD
	
	
	if(validConfig == 1) then
		configID = configID + 1 -- increment the configuration ID
		if(configID >= startID and configID <= stopID) then
			mi_deletematerial("WIRE_CUST") -- set the new wire gauge
			mi_addmaterial("WIRE_CUST",1,1,0,0,58,0,0,1,3,0,0,1,condOD[wireAWG-AWGIndexOffset]*25.4)
			
			-- update the progress label
			tempString = "Configuration # "..(configID-1)
			tempString2 = "Configuration # "..(configID)
			mi_modifymaterial(tempString,0,tempString2)
			
			mi_clearselected()
			mi_selectlabel(progressLabel[1],progressLabel[2])
			mi_setblockprop(tempString2,1,0,"<none>",0,0,0)
			
			magnetNodes = defineRectNodes (magID, magOD, magLen) 	-- Magnet ID, Magnet OD, Magnet Length
			defineCoilNodes   (coilID, coilOD, coilLen, coilCC)     -- Coil ID, Coil OD, Coil Length, Coil Spacing (total actuator travel)
			shieldNodes = defineRectNodes (coilOD+coil2shieldSpace*2, coilOD+coil2shieldSpace*2+shieldThickness*2, shieldLength) -- Define shield 
		
			--draw the magnet and coils
			mi_zoomnatural()
			--messagebox(magnetNodes)
			drawRect (magnetNodes, "MAG_CUST" ,"<none>", magnetGroup,       0, magnetLabel )
			--pause()
			mi_zoomnatural()
			drawRect (coilANodes,  "WIRE_CUST","COILA" , coilAGroup, numTurns, coilALabel  )
			--pause()
			drawRect (coilBNodes,  "WIRE_CUST","COILB" , coilBGroup, numTurns, coilBLabel  )
			--pause()
			drawRect (shieldNodes, "1020 Steel","<none>", shieldGroup,		0, shieldLabel )
			--pause()
		
			mi_modifycircprop("COILA",1, 1) -- Run both coils at 1 Amp for now, so that we can get the resistance after the first analysis
			mi_modifycircprop("COILB",1,-1) -- Run both coils at 1 Amp for now, so that we can get the resistance after the first analysis
		
			mi_analyze()		-- perform an initial analysis to get the coil resistance
			mi_loadsolution()	-- load the solution window
			
			mo_hidemesh()		 -- hopefully hiding the mesh will also speed up the process   - only need to do this once!
			mo_hidecontourplot() -- hopefully hiding the flux lines will speed up the process  - only need to do this once!
			mo_hidepoints()		 -- hopefully hiding the node points will speed up the process - only need to do this once!
			mo_minimize()
			
			testCurrent,testVoltage = mo_getcircuitproperties("COILA") 	-- get the voltage and current properties of the coil
			resistance = testVoltage/testCurrent 						-- determine the coil resistance
			
			amperes = voltage/resistance -- set the simulation coil current equal to the drive voltage/(coil resistance)
			if(forcedAmperes > 0) then amperes = forcedAmperes end
			if(forcedPower > 0) then amperes = sqrt(forcedPower/resistance) end
			mi_modifycircprop("COILA",1, amperes)  -- put the new ampere setting into the Coil A circuit
			mi_modifycircprop("COILB",1,-amperes)  -- put the new ampere setting into the Coil A circuit
			
		
			-- write the configuration parameters to the start of the entry line
			-- ConfigID,Voltage,Current,Resistance,Power,AWG,#Turns,Layers,RangeOfMotion,MagnetID,MagnetOD,MagnetLen,CoilID,CoilOD,CoilLen
			write(file,configID,",",voltage,",",amperes,",",resistance,",",voltage*amperes,",",wireAWG,",",numTurns,",",numLayers,",",coilCC,",",magID,",",magOD,",",magLen,",",coilID,",",coilOD,",",coilLen)
					
			mi_clearselected()
			mi_selectgroup(magnetGroup)
			mi_movetranslate(0,-magTotalTravel/2,4)	-- move the magnet and associated label to the start point
			magnetLabel[2] = magnetLabel[2]-magTotalTravel/2
			
			for magPos = -magTotalTravel/2, magTotalTravel/2, magTravelStep do
				write(file,",")
				if(magPos >= -coilCC/2-magOvershoot and magPos <= coilCC/2+magOvershoot) then
					mi_analyze()
					mi_loadsolution()
					mo_selectblock(magnetLabel[1],magnetLabel[2])
					write(file,mo_blockintegral(19))
				end
				--move the magnet for the next iteration
				mi_selectgroup(magnetGroup)
				mi_movetranslate(0,magTravelStep,4)
				magnetLabel[2] = magnetLabel[2]+magTravelStep
			end
			write(file,"\n")
			mi_selectgroup(magnetGroup)
			mi_deleteselected()
			--pause()
			mi_selectgroup(coilAGroup)
			mi_deleteselected()
			--pause()
			mi_selectgroup(coilBGroup)
			mi_deleteselected()
			--pause()
			mi_selectgroup(shieldGroup)
			mi_deleteselected()
			--pause()
			mo_close()
		end -- end IF between startID and stopID
		
	end	-- end of IF VALID CONFIGURATION
	coilOD = oldcoilOD
	

end end end end end end end end-- end of main loop block

closefile(file)